@@ -1,4 +1,4 @@
-#' Check if all Columns are Present
+#' Check if all columns are present
 #'
 #' `check_cols` creates a *specification* of a recipe
 #'  step that will check if all the columns of the training frame are
@@ -61,6 +61,7 @@ check_cols_new <-
     )
   }
 
+#' @export
 prep.check_cols <- function(x, training, info = NULL, ...) {
   col_names <- recipes_eval_select(x$terms, training, info)
 
@@ -74,19 +75,17 @@ prep.check_cols <- function(x, training, info = NULL, ...) {
   )
 }
 
+#' @export
 bake.check_cols <- function(object, new_data, ...) {
   original_cols <- object$columns
   new_cols <- names(new_data)
   missing <- setdiff(original_cols, new_cols)
   if (length(missing) > 0) {
-    mis_cols <- paste(paste0("`", missing, "`"), collapse = ", ")
-    rlang::abort(
-      paste0(
-        "The following cols are missing from `new_data`: ",
-        mis_cols,
-        "."
-      )
-    )
+    cli::cli_abort(c(
+      x = "{cli::qty(length(missing))}The following column{?s} {?is/are} \\
+      missing from {.arg new_data}:",
+      "*" = "{.and {.var {missing}}}."
+    ))
   }
   new_data
 }
