@@ -1,8 +1,8 @@
-#' High Correlation Filter
+#' High correlation filter
 #'
-#' `step_corr` creates a *specification* of a recipe
-#'  step that will potentially remove variables that have large
-#'  absolute correlations with other variables.
+#' `step_corr()` creates a *specification* of a recipe step that will
+#' potentially remove variables that have large absolute correlations with other
+#' variables.
 #'
 #' @inheritParams step_center
 #' @param threshold A value for the threshold of absolute
@@ -39,8 +39,19 @@
 #'
 #' # Tidying
 #'
-#' When you [`tidy()`][tidy.recipe()] this step, a tibble with column
-#' `terms` (the columns that will be removed) is returned.
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble is returned with
+#' columns `terms` and `id`:
+#'
+#' \describe{
+#'   \item{terms}{character, the selectors or variables selected to be removed}
+#'   \item{id}{character, id of this step}
+#' }
+#'
+#' ```{r, echo = FALSE, results="asis"}
+#' step <- "step_corr"
+#' result <- knitr::knit_child("man/rmd/tunable-args.Rmd")
+#' cat(result)
+#' ```
 #'
 #' @template case-weights-unsupervised
 #'
@@ -153,9 +164,7 @@ prep.step_corr <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_corr <- function(object, new_data, ...) {
-  if (length(object$removals) > 0) {
-    new_data <- new_data[, !(colnames(new_data) %in% object$removals)]
-  }
+  new_data <- recipes_remove_cols(new_data, object)
   new_data
 }
 
@@ -178,28 +187,25 @@ corr_filter <-
     if (any(!complete.cases(x))) {
       all_na <- apply(x, 2, function(x) all(is.na(x)))
       if (sum(all_na) >= nrow(x) - 1) {
-        rlang::warn("Too many correlations are `NA`; skipping correlation filter.")
+        cli::cli_warn(
+          "Too many correlations are `NA`; skipping correlation filter."
+        )
         return(numeric(0))
       } else {
         na_cols <- which(all_na)
         if (length(na_cols) > 0) {
           x[na_cols, ] <- 0
           x[, na_cols] <- 0
-          rlang::warn(
-            paste0(
-              "The correlation matrix has missing values. ",
-              length(na_cols),
-              " columns were excluded from the filter."
-            )
+          cli::cli_warn(
+            "The correlation matrix has missing values. \\
+            {length(na_cols)} column{?s} {?was/were} excluded from the filter."
           )
         }
       }
       if (any(is.na(x))) {
-        rlang::warn(
-          paste0(
-            "The correlation matrix has sporadic missing values. ",
-            "Some columns were excluded from the filter."
-          )
+        cli::cli_warn(
+          "The correlation matrix has sporadic missing values. \\
+          Some columns were excluded from the filter."
         )
         x[is.na(x)] <- 0
       }
