@@ -1,48 +1,7 @@
-filter_terms <- function(x, ...)
-  UseMethod("filter_terms")
-
-## Buckets variables into discrete, mutally exclusive types
-get_types <- function(x) {
-  var_types <-
-    c(
-      character = "nominal",
-      factor = "nominal",
-      ordered = "nominal",
-      integer = "numeric",
-      numeric = "numeric",
-      double = "numeric",
-      Surv = "censored",
-      logical = "logical",
-      Date = "date",
-      POSIXct = "date",
-      list = "list"
-    )
-
-  classes <- lapply(x, class)
-  res <- lapply(classes,
-                function(x, types) {
-                  in_types <- x %in% names(types)
-                  if (sum(in_types) > 0) {
-                    # not sure what to do with multiple matches; right now
-                    ## pick the first match which favors "factor" over "ordered"
-                    out <-
-                      unname(types[min(which(names(types) %in% x))])
-                  } else
-                    out <- "other"
-                  out
-                },
-                types = var_types)
-  res <- unlist(res)
-  tibble(variable = names(res), type = unname(res))
-}
-
-## get variables from formulas
-is_formula <- function(x)
-  isTRUE(inherits(x, "formula"))
-
 get_lhs_vars <- function(formula, data) {
-  if (!is_formula(formula))
+  if (!rlang::is_formula(formula)) {
     formula <- as.formula(formula)
+  }
   ## Want to make sure that multiple outcomes can be expressed as
   ## additions with no cbind business and that `.` works too (maybe)
   new_formula <- rlang::new_formula(lhs = NULL, rhs = f_lhs(formula))
@@ -50,7 +9,7 @@ get_lhs_vars <- function(formula, data) {
 }
 
 get_rhs_vars <- function(formula, data, no_lhs = FALSE) {
-  if (!is_formula(formula)) {
+  if (!rlang::is_formula(formula)) {
     formula <- as.formula(formula)
   }
   if (no_lhs) {
@@ -62,7 +21,7 @@ get_rhs_vars <- function(formula, data, no_lhs = FALSE) {
   ## or should it? what about Y ~ log(x)?
   ## Answer: when called from `form2args`, the function
   ## `inline_check` stops when in-line functions are used.
-  data_info <- attr(model.frame(formula, data[1,]), "terms")
+  data_info <- attr(model.frame(formula, data[1, ]), "terms")
   response_info <- attr(data_info, "response")
   predictor_names <- names(attr(data_info, "dataClasses"))
   if (length(response_info) > 0 && all(response_info > 0)) {
@@ -71,40 +30,6 @@ get_rhs_vars <- function(formula, data, no_lhs = FALSE) {
   predictor_names
 }
 
-get_lhs_terms <- function(x) x
-get_rhs_terms <- function(x) x
-
-
-
-terms.recipe <- function(x, ...)
-  x$term_info
-
-filter_terms.formula <- function(formula, data, ...)
-  get_rhs_vars(formula, data)
-
-
-## This function takes the default arguments of `func` and
-## replaces them with the matching ones in `options` and
-## remove any in `removals`
-sub_args <- function(func, options, removals = NULL) {
-  args <- formals(func)
-  for (i in seq_along(options))
-    args[[names(options)[i]]] <- options[[i]]
-  if (!is.null(removals))
-    args[removals] <- NULL
-  args
-}
-## Same as above but starts with a call object
-mod_call_args <- function(cl, args, removals = NULL) {
-  if (!is.null(removals))
-    for (i in removals)
-      cl[[i]] <- NULL
-    arg_names <- names(args)
-    for (i in arg_names)
-      cl[[i]] <- args[[i]]
-    cl
-}
-
 #' Naming Tools
 #'
 #' `names0` creates a series of `num` names with a common prefix.
@@ -134,13 +59,18 @@ mod_call_args <- function(cl, args, removals = NULL) {
 #' @return `names0` returns a character string of length `num` and
 #'  `dummy_names` generates a character vector the same length as
 #'  `lvl`.
+#'
+#' @seealso [developer_functions]
+#'
 #' @examples
 #' names0(9, "a")
 #' names0(10, "a")
 #'
-#' example <- data.frame(x = ordered(letters[1:5]),
-#'                       y = factor(LETTERS[1:5]),
-#'                       z = factor(paste(LETTERS[1:5], 1:5)))
+#' example <- data.frame(
+#'   x = ordered(letters[1:5]),
+#'   y = factor(LETTERS[1:5]),
+#'   z = factor(paste(LETTERS[1:5], 1:5))
+#' )
 #'
 #' dummy_names("y", levels(example$y)[-1])
 #' dummy_names("z", levels(example$z)[-1])
@@ -151,11 +81,11 @@ mod_call_args <- function(cl, args, removals = NULL) {
 #'
 #' dummy_names("x", substring(after_mm, 2), ordinal = TRUE)
 #' @export
-
 names0 <- function(num, prefix = "x") {
-  if (num < 1)
-    rlang::abort("`num` should be > 0")
-  ind <- format(1:num)
+  if (num < 1) {
+    cli::cli_abort("{.arg num} should be > 0.")
+  }
+  ind <- format(seq_len(num))
   ind <- gsub(" ", "0", ind)
   paste0(prefix, ind)
 }
@@ -168,45 +98,80 @@ dummy_names <- function(var, lvl, ordinal = FALSE, sep = "_") {
   var <- args[[1]]
   lvl <- args[[2]]
 
-  if(!ordinal)
+  if (!ordinal) {
     nms <- paste(var, make.names(lvl), sep = sep)
-  else
+  } else {
     # assuming they are in order:
     nms <- paste0(var, names0(length(lvl), sep))
+  }
 
   nms
 }
 
+#' @export
+#' @rdname names0
+dummy_extract_names <- function(var, lvl, ordinal = FALSE, sep = "_") {
+  # Work around `paste()` recycling bug with 0 length input
+  args <- vctrs::vec_recycle_common(var, lvl)
+  var <- args[[1]]
+  lvl <- args[[2]]
+
+  if (!ordinal) {
+    nms <- paste(var, make.names(lvl), sep = sep)
+  } else {
+    # assuming they are in order:
+    nms <- paste0(var, names0(length(lvl), sep))
+  }
+
+  while (any(duplicated(nms))) {
+    dupe_count <- vapply(seq_along(nms), function(i) {
+      sum(nms[i] == nms[1:i])
+    }, 1L)
+    nms[dupe_count > 1] <- paste(
+      nms[dupe_count > 1],
+      dupe_count[dupe_count > 1],
+      sep = sep
+    )
+  }
+  nms
+}
+
 
 ## As suggested by HW, brought in from the `pryr` package
 ## https://github.com/hadley/pryr
 fun_calls <- function(f) {
   if (is.function(f)) {
     fun_calls(body(f))
-  } else if (is_quosure(f))  {
+  } else if (is_quosure(f)) {
     fun_calls(quo_get_expr(f))
   } else if (is.call(f)) {
     fname <- as.character(f[[1]])
     # Calls inside .Internal are special and shouldn't be included
-    if (identical(fname, ".Internal"))
+    if (identical(fname, ".Internal")) {
       return(fname)
+    }
     unique(c(fname, unlist(lapply(f[-1], fun_calls), use.names = FALSE)))
   }
 }
 
 
 get_levels <- function(x) {
-  if (!is.factor(x) & !is.character(x))
+  if (!is.factor(x) & !is.character(x)) {
     return(list(values = NA, ordered = NA))
+  }
   out <-
     if (is.factor(x)) {
-      list(values = levels(x),
-           ordered = is.ordered(x),
-           factor = TRUE)
+      list(
+        values = levels(x),
+        ordered = is.ordered(x),
+        factor = TRUE
+      )
     } else {
-      list(values = sort(unique(x)),
-           ordered = FALSE,
-           factor = FALSE)
+      list(
+        values = sort(unique(x)),
+        ordered = FALSE,
+        factor = FALSE
+      )
     }
   out
 }
@@ -227,8 +192,9 @@ strings2factors <- function(x, info) {
     lcol <- names(info)[i]
     x[, lcol] <-
       factor(as.character(x[[lcol]]),
-             levels = info[[i]]$values,
-             ordered = info[[i]]$ordered)
+        levels = info[[i]]$values,
+        ordered = info[[i]]$ordered
+      )
   }
   x
 }
@@ -261,8 +227,10 @@ flatten_na <- function(x) {
 
 ## short summary of training set
 train_info <- function(x) {
-  data.frame(nrows = nrow(x),
-             ncomplete = n_complete_rows(x))
+  data.frame(
+    nrows = nrow(x),
+    ncomplete = n_complete_rows(x)
+  )
 }
 
 # ------------------------------------------------------------------------------
@@ -280,7 +248,7 @@ merge_term_info <- function(.new, .old) {
   # the original value
   .new %>%
     dplyr::rename(new_type = type) %>%
-    dplyr::left_join(.old, by = "variable") %>%
+    dplyr::left_join(.old, by = "variable", multiple = "all") %>%
     dplyr::mutate(
       type = ifelse(is.na(type), "other", "type"),
       type = ifelse(type != new_type, new_type, type)
@@ -291,6 +259,9 @@ merge_term_info <- function(.new, .old) {
 
 #' Check for Empty Ellipses
 #'
+#' `ellipse_check()` is deprecated. Instead, empty selections should be
+#' supported by all steps.
+#'
 #' @param ... Arguments pass in from a call to `step`
 #' @return If not empty, a list of quosures. If empty, an error is thrown.
 #' @export
@@ -298,13 +269,15 @@ merge_term_info <- function(.new, .old) {
 #' @rdname recipes-internal
 ellipse_check <- function(...) {
   terms <- quos(...)
-  if (is_empty(terms))
-    rlang::abort(
-      paste0(
-      "Please supply at least one variable specification.",
-      "See ?selections."
+  if (is_empty(terms)) {
+    cli::cli_abort(
+      c(
+        "!" = "Please supply at least one variable specification.",
+        "i" = "See {.help [?selections](recipes::selections)} \\
+              for more information."
       )
     )
+  }
   terms
 }
 
@@ -329,15 +302,22 @@ printer <- function(tr_obj = NULL,
                     width = max(20, options()$width - 30)) {
   if (trained) {
     txt <- format_ch_vec(tr_obj, width = width)
-  } else
+  } else {
     txt <- format_selectors(untr_obj, width = width)
-  if (nchar(txt) == 0)
+  }
+
+  if (length(txt) == 0L) {
     txt <- "<none>"
+  }
+
   cat(txt)
-  if (trained)
+
+  if (trained) {
     cat(" [trained]\n")
-  else
+  } else {
     cat("\n")
+  }
+
   invisible(NULL)
 }
 
@@ -345,9 +325,11 @@ printer <- function(tr_obj = NULL,
 #' @export
 #' @keywords internal
 #' @rdname recipes-internal
-prepare <- function(x, ...)
-  rlang::abort(paste0("As of version 0.0.1.9006, used `prep` ",
-       "instead of `prepare`"))
+prepare <- function(x, ...) {
+  cli::cli_abort(
+    "As of version 0.0.1.9006 please use {.fn prep} instead of {.fn prepare}."
+  )
+}
 
 
 #' Check to see if a recipe is trained/prepared
@@ -357,6 +339,9 @@ prepare <- function(x, ...)
 #'  through `prep`. If no steps have been added to the recipe, `TRUE` is
 #'  returned only if the recipe has been prepped.
 #' @export
+#'
+#' @seealso [developer_functions]
+#'
 #' @examples
 #' rec <- recipe(Species ~ ., data = iris) %>%
 #'   step_center(all_numeric())
@@ -364,7 +349,9 @@ prepare <- function(x, ...)
 #' rec %>% fully_trained()
 #'
 #'
-#' rec %>% prep(training = iris) %>% fully_trained()
+#' rec %>%
+#'   prep(training = iris) %>%
+#'   fully_trained()
 fully_trained <- function(x) {
   if (is.null(x$steps)) {
     if (any(names(x) == "last_term_info")) {
@@ -387,36 +374,29 @@ fully_trained <- function(x) {
 #' @return Logical indicating if recipes contains given step.
 #' @export
 #'
+#' @seealso [developer_functions]
+#'
 #' @examples
 #' rec <- recipe(Species ~ ., data = iris) %>%
 #'   step_intercept()
 #'
-#' detect_step(rec, "step_intercept")
+#' detect_step(rec, "intercept")
 detect_step <- function(recipe, name) {
-  exports <- getNamespaceExports("recipes")
-  if (!any(grepl(paste0(".*", name, ".*"), exports)))
-    rlang::abort("Please provide the name of valid step or check (ex: `center`).")
   name %in% tidy(recipe)$type
 }
 
 # to be used in a recipe
 is_skipable <- function(x) {
-  if(all("skip" != names(x)))
+  if (all("skip" != names(x))) {
     return(FALSE)
-  else
+  } else {
     return(x$skip)
+  }
 }
 
-# to be used within a step
-skip_me <- function(x) {
-  if(!exists("skip"))
-    return(FALSE)
-  else
-    return(x$skip)
-}
-
-is_qual <- function(x)
+is_qual <- function(x) {
   is.factor(x) | is.character(x)
+}
 
 #' Quantitatively check on variables
 #'
@@ -425,29 +405,79 @@ is_qual <- function(x)
 #'   check fails.
 #' @param dat A data frame or tibble of the training data.
 #' @param quant A logical indicating whether the data is expected to be numeric
-#'   (TRUE) or a factor/character (FALSE).
+#'   (TRUE) or a factor/character (FALSE). Is ignored if `types` is specified.
+#' @param types Character vector of allowed types. Following the same types as
+#'   [has_role()]. See details for more.
+#'
+#' @details
+#' Using `types` is a more fine-tuned way to use this. function compared to using
+#' `quant`. `types` should specify all allowed types as designated by
+#' [.get_data_types]. Suppose you want to allow doubles, integers, characters,
+#' factors and ordered factors, then you should specify
+#' `types = c("double", "integer", "string", "factor", "ordered")` to get a
+#' clear error message.
+#'
+#' @seealso [developer_functions]
+#'
 #' @export
 #' @keywords internal
-check_type <- function(dat, quant = TRUE) {
-  if (quant) {
-    all_good <- vapply(dat, is.numeric, logical(1))
-    label <- "numeric"
+check_type <- function(dat, quant = TRUE, types = NULL, call = caller_env()) {
+  if (is.null(types)) {
+    if (quant) {
+      all_good <- vapply(dat, is.numeric, logical(1))
+      types <- "numeric"
+    } else {
+      all_good <- vapply(dat, is_qual, logical(1))
+      types <- "factor or character"
+    }
   } else {
-    all_good <- vapply(dat, is_qual, logical(1))
-    label <- "factor or character"
+    all_good <- purrr::map_lgl(get_types(dat)$type, ~ any(.x %in% types))
   }
-  if (!all(all_good))
-    rlang::abort(
-      paste0(
-        "All columns selected for the step",
-         " should be ",
-         label)
-      )
+
+  if (!all(all_good)) {
+    info <- get_types(dat)[!all_good, ]
+    classes <- map_chr(info$type, function(x) x[1])
+    counts <- vctrs::vec_split(info$variable, classes)
+    counts$count <- lengths(counts$val)
+    counts$text_len <- cli::console_width() - 18 - (counts$count > 1) -
+      nchar(counts$key) - (counts$count > 2)
+
+    # cli::ansi_collapse() doesn't work for length(x) == 1
+    # https://github.com/r-lib/cli/issues/590
+    variable_collapse <- function(x, width) {
+      x <- paste0("{.var ", x, "}")
+      if (length(x) == 1) {
+        res <- cli::ansi_strtrim(x, width = width)
+      } else if (length(x) == 2) {
+        res <- cli::ansi_collapse(
+          x, last = " and ", width = width, style = "head"
+        )
+      } else {
+        res <- cli::ansi_collapse(
+          x, width = width, style = "head"
+        )
+      }
+      res
+    }
+
+    problems <- glue::glue_data(
+      counts,
+      "{count} {key} variable{ifelse(count == 1, '', 's')} \\
+      found: {purrr::map2_chr(val, text_len, variable_collapse)}"
+    )
+    names(problems) <- rep("*", length(problems))
+
+    message <- "All columns selected for the step should be {.or {types}}."
+
+    cli::cli_abort(
+      c("x" = message, problems),
+      call = call
+    )
+  }
+
   invisible(all_good)
 }
 
-
-
 ## Support functions
 
 #' Check to see if a step or check as been trained
@@ -456,8 +486,10 @@ check_type <- function(dat, quant = TRUE) {
 #' @export
 #' @keywords internal
 #' @rdname recipes-internal
-is_trained <- function(x)
+#' @seealso [developer_functions]
+is_trained <- function(x) {
   x$trained
+}
 
 
 #' Convert Selectors to Character
@@ -470,6 +502,7 @@ is_trained <- function(x)
 #' @export
 #' @keywords internal
 #' @rdname recipes-internal
+#' @seealso [developer_functions]
 sel2char <- function(x) {
   unname(map_chr(x, to_character))
 }
@@ -506,26 +539,32 @@ simple_terms <- function(x, ...) {
 #'   in the trained object.
 #' @param names A logical determining if the names should be set using
 #' the names function (TRUE) or colnames function (FALSE).
+#' @param call The execution environment of a currently running function, e.g.
+#'   `caller_env()`. The function will be mentioned in error messages as the
+#'   source of the error. See the call argument of [rlang::abort()] for more
+#'   information.
+#'
+#' @seealso [developer_functions]
+#'
 #' @export
 #' @keywords internal
-check_name <- function(res, new_data, object, newname = NULL, names = FALSE) {
-  if(is.null(newname)) {
+check_name <- function(res, new_data, object, newname = NULL, names = FALSE,
+                       call = caller_env()) {
+  if (is.null(newname)) {
     newname <- names0(ncol(res), object$prefix)
   }
   new_data_names <- colnames(new_data)
   intersection <- new_data_names %in% newname
-  if(any(intersection)) {
-    rlang::abort(
-      paste0(
-        "Name collision occured in `",
-        class(object)[1],
-        "`. The following variable names already exists: ",
-        paste0(new_data_names[intersection], collapse = ", "),
-        "."
-      )
+  if (any(intersection)) {
+    nms <- new_data_names[intersection]
+    cli::cli_abort(
+      c("Name collision occurred. The following variable names already exist:",
+        "*" = "{.var {nms}}"),
+      call = call
     )
+
   }
-  if(names) {
+  if (names) {
     names(res) <- newname
   } else {
     colnames(res) <- newname
@@ -535,17 +574,20 @@ check_name <- function(res, new_data, object, newname = NULL, names = FALSE) {
 
 #' Make a random identification field for steps
 #'
+#'
 #' @export
 #' @param prefix A single character string
 #' @param len An integer for the number of random characters
 #' @return A character string with the prefix and random letters separated by
 #'  and underscore.
+#'
+#' @seealso [developer_functions]
 #' @keywords internal
 rand_id <- function(prefix = "step", len = 5) {
   candidates <- c(letters, LETTERS, paste(0:9))
   paste(prefix,
-        paste0(sample(candidates, len, replace = TRUE), collapse = ""),
-        sep = "_"
+    paste0(sample(candidates, len, replace = TRUE), collapse = ""),
+    sep = "_"
   )
 }
 
@@ -575,17 +617,13 @@ check_nominal_type <- function(x, lvl) {
     was_factor <- fac_ref_cols[!(fac_ref_cols %in% fac_act_cols)]
 
     if (length(was_factor) > 0) {
-      rlang::warn(
-        paste0(
-          " There ",
-          ifelse(length(was_factor) > 1, "were ", "was "),
-          length(was_factor),
-          ifelse(length(was_factor) > 1, " columns ", " column "),
-          "that ",
-          ifelse(length(was_factor) > 1, "were factors ", "was a factor "),
-          "when the recipe was prepped:\n ",
-          paste0("'", was_factor, "'", collapse = ", "),
-          ".\n This may cause errors when processing new data."
+      cli::cli_warn(
+        c(
+          "!" = "There {?w/was/were} {length(was_factor)} column{?s} that \\
+                {?was a factor/were factors} when the recipe was prepped: \\
+                ",
+          "*" = "{.and {.var {was_factor}}}",
+          "i" = "This may cause errors when processing new data."
         )
       )
     }
@@ -593,27 +631,27 @@ check_nominal_type <- function(x, lvl) {
   invisible(NULL)
 }
 
-check_training_set <- function(x, rec, fresh) {
+check_training_set <- function(x, rec, fresh, call = rlang::caller_env()) {
   # In case a variable has multiple roles
   vars <- unique(rec$var_info$variable)
 
-  if (is.null(x)) {
-    if (fresh)
-      rlang::abort(
-        paste0("A training set must be supplied to the `training` argument ",
-               "when `fresh = TRUE`."
-        )
+  training_null <- is.null(x)
+  if (training_null) {
+    if (fresh) {
+      cli::cli_abort(
+        "A training set must be supplied to the {.arg training} argument \\
+        when {.code fresh = TRUE}.",
+        call = call
       )
+    }
     x <- rec$template
   } else {
     in_data <- vars %in% colnames(x)
     if (!all(in_data)) {
-      rlang::abort(
-        paste0("Not all variables in the recipe are present in the supplied ",
-               "training set: ",
-               paste0("'", vars[!in_data], "'", collapse = ", "),
-               "."
-        )
+      cli::cli_abort(
+        "Not all variables in the recipe are present in the supplied training \\
+        set: {.and {.var {vars[!in_data]}}}.",
+        call = call
       )
     }
     if (!is_tibble(x)) {
@@ -625,22 +663,21 @@ check_training_set <- function(x, rec, fresh) {
 
   steps_trained <- vapply(rec$steps, is_trained, logical(1))
   if (any(steps_trained) & !fresh) {
-    if(!rec$retained) {
-      rlang::abort(
-        paste0(
-          "To prep new steps after prepping the original ",
-          "recipe, `retain = TRUE` must be set each time that ",
-          "the recipe is trained."
-        )
+    if (!rec$retained) {
+      cli::cli_abort(
+        "To prep new steps after prepping the original recipe, \\
+        {.code retain = TRUE} must be set each time that the recipe is \\
+        trained.",
+        call = call
       )
     }
-    if (!is.null(rec$training)) {
-      rlang::warn(
-        paste0(
-          "The previous data will be used by `prep`; ",
-          "the data passed using `training` will be ",
-          "ignored."
-        )
+    if (!training_null) {
+      cli::cli_warn(
+        c(
+          "!" = "The previous data will be used by {.fn prep}.",
+          "i" = "The data passed using {.arg training} will be ignored."
+        ),
+        call = call
       )
     }
     x <- rec$template
@@ -654,18 +691,18 @@ check_training_set <- function(x, rec, fresh) {
 #' @param object A recipe step
 #' @return A logical to keep the original variables in the output
 #' @keywords internal
+#' @seealso [developer_functions]
 get_keep_original_cols <- function(object) {
   # Allow prepping of old recipes created before addition of keep_original_cols
   step_class <- class(object)[1]
 
   if (is.null(object$keep_original_cols)) {
     ret <- FALSE
-    rlang::warn(
-      paste0(
-        "'keep_original_cols' was added to `",
-        step_class,
-        "()` after this recipe was created.\n",
-        "Regenerate your recipe to avoid this warning."
+    cli::cli_warn(
+      c(
+        "{.arg keep_original_cols} was added to {.fn {step_class}} after this \\
+         recipe was created.",
+        "i" = "Regenerate your recipe to avoid this warning."
       )
     )
   } else {
@@ -676,19 +713,6 @@ get_keep_original_cols <- function(object) {
 }
 
 # ------------------------------------------------------------------------------
-# From parsnip, keep synced
-
-is_varying <- function(x) {
-  if (is.null(x)) {
-    res <- FALSE
-  } else {
-    res <- if (is_quosure(x))
-      isTRUE(all.equal(x[[-1]], quote(varying())))
-    else
-      isTRUE(all.equal(x, quote(varying())))
-  }
-  res
-}
 
 # from tune package
 is_tune <- function(x) {
@@ -704,6 +728,7 @@ is_tune <- function(x) {
   FALSE
 }
 
+
 # ------------------------------------------------------------------------------
 # For all imputation functions that substitute elements into an existing vector:
 # vctrs's cast functions would be better but we'll deal with the known cases
@@ -732,7 +757,7 @@ print_col_names <- function(x, prefix = "") {
   }
   nm_sdth <- cumsum(c(nchar(prefix), purrr::map_int(x, nchar) + 2))
   keep_x <- nm_sdth + 5 < wdth
-  x <- x[ keep_x[-1] ]
+  x <- x[keep_x[-1]]
   y <- paste0(prefix, paste0(x, collapse = ", "))
   if (!all(keep_x)) {
     y <- paste0(y, ", ...")
@@ -754,8 +779,162 @@ changelog <- function(show, before, after, x) {
   } else {
     cat("\n")
     print_col_names(new_cols, " new")
-    print_col_names(rm_cols,  " removed")
+    print_col_names(rm_cols, " removed")
     cat("\n")
   }
 }
 
+# ------------------------------------------------------------------------------
+
+eval_dimred_call <- function(fn, ...) {
+  cl <- rlang::call2(fn, .ns = "dimRed", ...)
+  rlang::eval_tidy(cl)
+}
+
+dimred_data <- function(dat) {
+  cl <- rlang::call2("dimRedData", .ns = "dimRed", rlang::expr(as.matrix(dat)))
+  rlang::eval_tidy(cl)
+}
+
+uses_dim_red <- function(x) {
+  dr <- inherits(x, "dimRedResult")
+  if (dr) {
+    cli::cli_abort(
+      "Recipes version >= 0.1.17 represents the estimates using a different \\
+      format. Please recreate this recipe or use version 0.1.16 or less. See \\
+      issue {.href [#823](https://github.com/tidymodels/recipes/issues/823)}."
+    )
+  }
+  invisible(NULL)
+}
+
+# ------------------------------------------------------------------------------
+
+#' Check for required column at bake-time
+#'
+#' When baking a step, create an information error message when a column that
+#' is used by the step is not present in `new_data`.
+#'
+#' @param req A character vector of required columns.
+#' @param object A step object.
+#' @param new_data A tibble of data being baked.
+#' @return Invisible NULL. Side effects are the focus of the function.
+#' @keywords internal
+#'
+#' @seealso [developer_functions]
+#'
+#' @export
+check_new_data <- function(req, object, new_data) {
+  if (is.null(req) || length(req) == 0L) {
+    return(invisible(NULL))
+  }
+  col_diff <- setdiff(req, names(new_data))
+  if (length(col_diff) == 0) {
+    return(invisible(NULL))
+  }
+  step_cls <- class(object)[1]
+  step_id <- object$id
+  cli::cli_abort(
+    "The following required {cli::qty(col_diff)} column{?s} {?is/are} \
+    missing from `new_data` in step '{step_id}': {col_diff}.",
+    class = "new_data_missing_column",
+    call = rlang::call2(step_cls)
+  )
+}
+
+stop_recipes <- function(class = NULL,
+                         call = NULL,
+                         parent = NULL) {
+  rlang::abort(
+    class = c(class, "recipes_error"),
+    call = call,
+    parent = parent
+  )
+}
+
+stop_recipes_step <- function(call = NULL,
+                              parent = NULL) {
+  stop_recipes(
+    class = "recipes_error_step",
+    call = call,
+    parent = parent
+  )
+}
+
+recipes_error_context <- function(expr, step_name) {
+  withCallingHandlers(
+    expr = force(expr),
+    error = function(cnd) {
+      stop_recipes_step(
+        call = call(step_name),
+        parent = cnd
+      )
+    }
+  )
+}
+
+#' @method conditionMessage recipes_error
+#' @export
+conditionMessage.recipes_error <- function(c) {
+  rlang::cnd_message(c, prefix = TRUE)
+}
+
+vec_paste0 <- function(..., collapse = NULL) {
+  args <- vctrs::vec_recycle_common(...)
+  rlang::inject(paste0(!!!args, collapse = collapse))
+}
+
+#' Removes original columns if options apply
+#'
+#' This helper function should be used whenever the argument
+#' `keep_original_cols` is used in a function.
+#'
+#' @param new_data A tibble.
+#' @param object A step object.
+#' @param col_names A character vector, denoting columns to remove.
+#' @return new_data with `col_names` removed if
+#'     `get_keep_original_cols(object) == TRUE` or `object$preserve == TRUE`.
+#' @keywords internal
+#'
+#' @seealso [developer_functions]
+#'
+#' @export
+remove_original_cols <- function(new_data, object, col_names) {
+  keep_original_cols <- get_keep_original_cols(object)
+  if (any(isFALSE(object$preserve), !keep_original_cols)) {
+    new_data <- recipes_remove_cols(new_data, object, col_names)
+  }
+  new_data
+}
+
+#' Removes columns if options apply
+#'
+#' This helper function removes columns based on character vectors.
+#'
+#' @param new_data A tibble.
+#' @param object A step object.
+#' @param col_names A character vector, denoting columns to remove. Will
+#'   overwrite `object$removals` if set.
+#'
+#' @return `new_data` with column names removed if specified by `col_names` or
+#'   `object$removals`.
+#' @keywords internal
+#'
+#' @seealso [developer_functions]
+#'
+#' @export
+recipes_remove_cols <- function(new_data, object, col_names = character()) {
+  if (length(col_names) > 0) {
+    removals <- col_names
+  } else if (length(object$removals) > 0) {
+    removals <- object$removals
+  } else {
+    return(new_data)
+  }
+
+  if (length(removals) > 0) {
+    # drop = FALSE in case someone uses this on a data.frame
+    new_data <- new_data[, !(colnames(new_data) %in% removals), drop = FALSE]
+  }
+  new_data
+}
