@@ -1,15 +1,16 @@
-#' NNMF Signal Extraction
+#' Non-negative matrix factorization signal extraction
 #'
-#' `step_nnmf` creates a *specification* of a recipe step
-#'  that will convert numeric data into one or more non-negative
-#'  components.
+#' @description
+#'
+#' `step_nnmf()` creates a *specification* of a recipe step that will convert
+#' numeric data into one or more non-negative components.
+#'
+#' `r lifecycle::badge("deprecated")`
+#'
+#' Please use [step_nnmf_sparse()] instead of this step function.
 #'
 #' @inheritParams step_pca
 #' @inheritParams step_center
-#' @param num_comp The number of components to retain as new
-#'  predictors. If `num_comp` is greater than the number of columns
-#'  or the number of possible components, a smaller value will be
-#'  used.
 #' @param num_run A positive integer for the number of computations runs used
 #'  to obtain a consensus projection.
 #' @param options A list of options to `nmf()` in the NMF package by way of the
@@ -18,7 +19,7 @@
 #'  processing is turned off in favor of resample-level parallelization.
 #' @param res The `NNMF()` object is stored
 #'  here once this preprocessing step has been trained by
-#'  [prep.recipe()].
+#'  [prep()].
 #' @param prefix A character string that will be the prefix to the
 #'  resulting new variables. See notes below.
 #' @param seed An integer that will be used to set the seed in isolation
@@ -30,22 +31,34 @@
 #'  have non-negative values and take into account that the original data
 #'  have non-negative values.
 #'
-#' The argument `num_comp` controls the number of components that
-#'  will be retained (the original variables that are used to derive
-#'  the components are removed from the data). The new components
-#'  will have names that begin with `prefix` and a sequence of
-#'  numbers. The variable names are padded with zeros. For example,
-#'  if `num < 10`, their names will be `NNMF1` - `NNMF9`.
-#'  If `num = 101`, the names would be `NNMF001` -
-#'  `NNMF101`.
+#' ```{r, echo = FALSE, results="asis"}
+#' prefix <- "NNMF"
+#' result <- knitr::knit_child("man/rmd/num_comp.Rmd")
+#' cat(result)
+#' ```
 #'
-#' When you [`tidy()`] this step, a tibble with column `terms` (the
-#'  selectors or variables selected) and the number of components is returned.
+#' # Tidying
 #'
-#' @examples
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble is returned with
+#' columns `terms`, `value`, `component` , and `id`:
 #'
-#' library(modeldata)
-#' data(biomass)
+#' \describe{
+#'   \item{terms}{character, the selectors or variables selected}
+#'   \item{value}{numeric, value of loading}
+#'   \item{component}{character, name of component}
+#'   \item{id}{character, id of this step}
+#' }
+#'
+#' ```{r, echo = FALSE, results="asis"}
+#' step <- "step_nnmf"
+#' result <- knitr::knit_child("man/rmd/tunable-args.Rmd")
+#' cat(result)
+#' ```
+#'
+#' @template case-weights-not-supported
+#'
+#' @examplesIf rlang::is_installed(c("modeldata", "ggplot2"))
+#' data(biomass, package = "modeldata")
 #'
 #' # rec <- recipe(HHV ~ ., data = biomass) %>%
 #' #   update_role(sample, new_role = "id var") %>%
@@ -63,27 +76,29 @@ step_nnmf <-
            ...,
            role = "predictor",
            trained = FALSE,
-           num_comp  = 2,
+           num_comp = 2,
            num_run = 30,
            options = list(),
            res = NULL,
+           columns = NULL,
            prefix = "NNMF",
            seed = sample.int(10^5, 1),
            keep_original_cols = FALSE,
            skip = FALSE,
-           id = rand_id("nnmf")
-           ) {
+           id = rand_id("nnmf")) {
     recipes_pkg_check(required_pkgs.step_nnmf())
+    lifecycle::deprecate_warn("0.2.0", "step_nnmf()", "step_nnmf_sparse()")
     add_step(
       recipe,
       step_nnmf_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         num_comp = num_comp,
         num_run = num_run,
         options = options,
         res = res,
+        columns = columns,
         prefix = prefix,
         seed = seed,
         keep_original_cols = keep_original_cols,
@@ -94,7 +109,7 @@ step_nnmf <-
   }
 
 step_nnmf_new <-
-  function(terms, role, trained, num_comp, num_run, options, res,
+  function(terms, role, trained, num_comp, num_run, options, res, columns,
            prefix, seed, keep_original_cols, skip, id) {
     step(
       subclass = "nnmf",
@@ -105,6 +120,7 @@ step_nnmf_new <-
       num_run = num_run,
       options = options,
       res = res,
+      columns = columns,
       prefix = prefix,
       seed = seed,
       keep_original_cols = keep_original_cols,
@@ -116,29 +132,35 @@ step_nnmf_new <-
 #' @export
 prep.step_nnmf <- function(x, training, info = NULL, ...) {
   col_names <- recipes_eval_select(x$terms, training, info)
+  check_type(training[, col_names], types = c("double", "integer"))
 
-  check_type(training[, col_names])
-
-  if (x$num_comp > 0) {
-
+  if (x$num_comp > 0 && length(col_names) > 0) {
     x$num_comp <- min(x$num_comp, length(col_names))
 
-    opts <- list(options = x$options)
-    opts$ndim <- x$num_comp
-    opts$nrun <- x$num_run
-    opts$seed <- x$seed
-    opts$.mute <- c("message", "output")
-    opts$.data <- dimRed::dimRedData(as.data.frame(training[, col_names, drop = FALSE]))
-    opts$.method <- "NNMF"
     nmf_opts <- list(parallel = FALSE, parallel.required = FALSE)
-    opts$options <- list(.options = nmf_opts)
 
-    nnm <- try(do.call(dimRed::embed, opts), silent = TRUE)
+    nnm <- try(
+      eval_dimred_call(
+        "embed",
+        .method = "NNMF",
+        .data = dimred_data(training[, col_names, drop = FALSE]),
+        ndim = x$num_comp,
+        nrun = x$num_run,
+        seed = x$seed,
+        .mute = c("message", "output"),
+        options = x$options,
+        .options = nmf_opts
+      ),
+      silent = TRUE
+    )
     if (inherits(nnm, "try-error")) {
-      rlang::abort(paste0("`step_nnmf` failed with error:\n", as.character(nnm)))
+      cli::cli_abort(c(
+        x = "Failed with error:",
+        i = as.character(nnm)
+      ))
     }
   } else {
-    nnm <- list(x_vars = col_names)
+    nnm <- NULL
   }
 
   step_nnmf_new(
@@ -149,6 +171,7 @@ prep.step_nnmf <- function(x, training, info = NULL, ...) {
     num_run = x$num_run,
     options = x$options,
     res = nnm,
+    columns = col_names,
     prefix = x$prefix,
     seed = x$seed,
     keep_original_cols = get_keep_original_cols(x),
@@ -159,34 +182,29 @@ prep.step_nnmf <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_nnmf <- function(object, new_data, ...) {
-  if (object$num_comp > 0) {
-    nnmf_vars <- rownames(object$res@other.data$w)
-    comps <-
-      object$res@apply(
-        dimRed::dimRedData(
-          as.data.frame(new_data[, nnmf_vars, drop = FALSE])
-        )
-      )@data
-    comps <- comps[, 1:object$num_comp, drop = FALSE]
-    colnames(comps) <- names0(ncol(comps), object$prefix)
-    new_data <- bind_cols(new_data, as_tibble(comps))
-    keep_original_cols <- get_keep_original_cols(object)
+  check_new_data(object$columns, object, new_data)
 
-    if (!keep_original_cols) {
-      new_data <- new_data[, !(colnames(new_data) %in% nnmf_vars), drop = FALSE]
-    }
+  keep_going <- object$num_comp > 0 && length(object$columns) > 0
+  if (!keep_going) {
+    return(new_data)
   }
-  as_tibble(new_data)
+
+  nnmf_vars <- rownames(object$res@other.data$w)
+  comps <-
+    object$res@apply(dimred_data(new_data[, nnmf_vars, drop = FALSE]))@data
+  comps <- comps[, seq_len(object$num_comp), drop = FALSE]
+  colnames(comps) <- names0(ncol(comps), object$prefix)
+  comps <- as_tibble(comps)
+  comps <- check_name(comps, new_data, object)
+  new_data <- vec_cbind(new_data, comps)
+  new_data <- remove_original_cols(new_data, object, nnmf_vars)
+  new_data
 }
 
 
 print.step_nnmf <- function(x, width = max(20, options()$width - 29), ...) {
-  if (x$num_comp == 0) {
-    cat("Non-negative matrix factorization was not done.\n")
-  } else {
-    cat("Non-negative matrix factorization for ")
-    printer(colnames(x$res@org.data), x$terms, x$trained, width = width)
-  }
+  title <- "Non-negative matrix factorization for "
+  print_step(colnames(x$res@org.data), x$terms, x$trained, title, width)
   invisible(x)
 }
 
@@ -195,21 +213,23 @@ print.step_nnmf <- function(x, width = max(20, options()$width - 29), ...) {
 #' @export
 tidy.step_nnmf <- function(x, ...) {
   if (is_trained(x)) {
-    if (x$num_comp > 0) {
+    if (x$num_comp > 0 && length(x$columns) > 0) {
       res <- x$res@other.data$w
       var_nms <- rownames(res)
       res <- tibble::as_tibble(res)
       res$terms <- var_nms
-      res <- tidyr::pivot_longer(res, cols = c(-terms),
-                                 names_to = "component", values_to = "value")
-      res <- res[,c("terms", "value", "component")]
-      res <- res[order(res$component, res$terms),]
+      res <- tidyr::pivot_longer(res,
+        cols = c(-terms),
+        names_to = "component", values_to = "value"
+      )
+      res <- res[, c("terms", "value", "component")]
+      res <- res[order(res$component, res$terms), ]
     } else {
-      res <- tibble(terms = x$res$x_vars, value = na_dbl, component  = na_chr)
+      res <- tibble(terms = unname(x$columns), value = na_dbl, component = na_dbl)
     }
   } else {
     term_names <- sel2char(x$terms)
-    res <- tibble(terms = term_names, value = na_dbl, component  = x$num_comp)
+    res <- tibble(terms = term_names, value = na_dbl, component = x$num_comp)
   }
   res$id <- x$id
   res
@@ -217,14 +237,13 @@ tidy.step_nnmf <- function(x, ...) {
 
 # ------------------------------------------------------------------------------
 
-#' @rdname tunable.recipe
 #' @export
 tunable.step_nnmf <- function(x, ...) {
   tibble::tibble(
     name = c("num_comp", "num_run"),
     call_info = list(
       list(pkg = "dials", fun = "num_comp", range = c(1L, 4L)),
-      list(pkg = "dials", fun = "num_run", range = c(1L, 10L))
+      list(pkg = "dials", fun = "num_runs", range = c(1L, 10L))
     ),
     source = "recipe",
     component = "step_nnmf",
@@ -237,4 +256,3 @@ tunable.step_nnmf <- function(x, ...) {
 required_pkgs.step_nnmf <- function(x, ...) {
   c("dimRed", "NMF")
 }
-
