@@ -1,8 +1,8 @@
 #' Center and scale numeric data
 #'
-#' `step_normalize` creates a *specification* of a recipe
-#'  step that will normalize numeric data to have a standard
-#'  deviation of one and a mean of zero.
+#' `step_normalize()` creates a *specification* of a recipe step that will
+#' normalize numeric data to have a standard deviation of one and a mean of
+#' zero.
 #'
 #' @inheritParams step_center
 #' @param means A named numeric vector of means. This is `NULL` until computed
@@ -21,12 +21,17 @@
 #'  `prep.recipe`. [`bake.recipe`] then applies the scaling to new data sets using
 #'  these estimates.
 #'
-#'  # Tidying
+#' # Tidying
 #'
-#'  When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
-#'  `terms` (the selectors or variables selected), `value` (the standard
-#'  deviations and means), and `statistic` for the type of value is
-#'  returned.
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble is returned with
+#' columns `terms`, `statistic`, `value` , and `id`:
+#'
+#' \describe{
+#'   \item{terms}{character, the selectors or variables selected}
+#'   \item{statistic}{character, name of statistic (`"mean"` or `"sd"`)}
+#'   \item{value}{numeric, value of the `statistic`}
+#'   \item{id}{character, id of this step}
+#' }
 #'
 #' @template case-weights-unsupervised
 #'
@@ -106,17 +111,27 @@ step_normalize_new <-
 sd_check <- function(x) {
   zero_sd <- which(x < .Machine$double.eps)
   if (length(zero_sd) > 0) {
-    glue_cols <- glue::glue_collapse(
-      glue::glue("`{names(zero_sd)}`"), sep = ", ", last = " and "
-    )
-    rlang::warn(
-      glue::glue(
-        "Column(s) have zero variance so scaling cannot be used: {glue_cols}. ",
-        "Consider using `step_zv()` to remove those columns before normalizing"
-      )
-    )
+    offenders <- names(zero_sd)
+
+    cli::cli_warn(c(
+      "!" = "{cli::qty(offenders)} The following column{?s} {?has/have} zero \\
+            variance so scaling cannot be used: {offenders}.",
+      "i" = "Consider using {.help [?step_zv](recipes::step_zv)} to remove \\
+            those columns before normalizing."
+    ))
+
     x[zero_sd] <- 1
   }
+
+  na_sd <- which(is.na(x))
+  if (length(na_sd) > 0) {
+    cli::cli_warn(
+        "Column{?s} {.var {names(na_sd)}} returned NaN, because variance \\
+        cannot be calculated and scaling cannot be used. Consider avoiding \\
+        `Inf` or `-Inf` values and/or setting `na_rm = TRUE` before \\
+        normalizing."
+    )
+  }
   x
 }
 
@@ -151,13 +166,16 @@ prep.step_normalize <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_normalize <- function(object, new_data, ...) {
-  check_new_data(names(object$means), object, new_data)
+  col_names <- names(object$means)
+  check_new_data(col_names, object, new_data)
 
-  for (column in names(object$means)) {
-    mean <- object$means[column]
-    sd <- object$sds[column]
-    new_data[[column]] <- (new_data[[column]] - mean) / sd
+  for (col_name in col_names) {
+    mean <- object$means[col_name]
+    sd <- object$sds[col_name]
+
+    new_data[[col_name]] <- (new_data[[col_name]] - mean) / sd
   }
+
   new_data
 }
 
