@@ -1,13 +1,10 @@
-#' Convert Ordinal Factors to Numeric Scores
+#' Convert ordinal factors to numeric scores
 #'
-#' `step_ordinalscore` creates a *specification* of a
-#'  recipe step that will convert ordinal factor variables into
-#'  numeric scores.
+#' `step_ordinalscore()` creates a *specification* of a recipe step that will
+#' convert ordinal factor variables into numeric scores.
 #'
 #' @inheritParams step_center
-#' @param columns A character string of variables that will be
-#'  converted. This is `NULL` until computed by
-#'  [prep()].
+#' @inheritParams step_pca
 #' @param convert A function that takes an ordinal factor vector
 #'  as an input and outputs a single numeric variable.
 #' @template step-return
@@ -21,10 +18,15 @@
 #'  a linear scale (1, 2, 3, ... `C`) but custom score
 #'  functions can also be used (see the example below).
 #'
-#'  # Tidying
+#' # Tidying
 #'
-#' When you [`tidy()`][tidy.recipe()] this step, a tibble with column
-#' `terms` (the columns that will be affected) is returned.
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble is returned with
+#' columns `terms` and `id`:
+#'
+#' \describe{
+#'   \item{terms}{character, the selectors or variables selected}
+#'   \item{id}{character, id of this step}
+#' }
 #'
 #' @template case-weights-not-supported
 #'
@@ -121,14 +123,15 @@ prep.step_ordinalscore <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_ordinalscore <- function(object, new_data, ...) {
-  check_new_data(object$columns, object, new_data)
+  col_names <- object$columns
+  check_new_data(col_names, object, new_data)
 
-  scores <- lapply(new_data[, object$columns], object$convert)
-  scores <- lapply(scores, vec_cast, integer())
-
-  for (i in object$columns) {
-    new_data[, i] <- scores[[i]]
+  for (col_name in col_names) {
+    score <- object$convert(new_data[[col_name]])
+    score <- vctrs::vec_cast(score, integer())
+    new_data[[col_name]] <- score
   }
+
   new_data
 }
 
