@@ -1,7 +1,7 @@
-#' PCA Signal Extraction
+#' PCA signal extraction
 #'
-#' `step_pca` creates a *specification* of a recipe step that will convert
-#'  numeric data into one or more principal components.
+#' `step_pca()` creates a *specification* of a recipe step that will convert
+#' numeric variables into one or more principal components.
 #'
 #' @inheritParams step_center
 #' @param role For model terms created by this step, what analysis role should
@@ -11,7 +11,7 @@
 #'  If `num_comp` is greater than the number of columns or the number of
 #'  possible components, a smaller value will be used. If `num_comp = 0`
 #'  is set then no transformation is done and selected variables will
-#'  stay unchanged.
+#'  stay unchanged, regardless of the value of `keep_original_cols`.
 #' @param threshold A fraction of the total variance that should be covered by
 #'  the components. For example, `threshold = .75` means that `step_pca` should
 #'  generate enough components to capture 75 percent of the variability in the
@@ -23,8 +23,8 @@
 #'  should not be passed here (or at all).
 #' @param res The [stats::prcomp.default()] object is stored here once this
 #'  preprocessing step has be trained by [prep()].
-#' @param columns A character string of variable names that will
-#'  be populated elsewhere.
+#' @param columns A character string of the selected variable names. This field
+#'   is a placeholder and will be populated once [prep()] is used.
 #' @param prefix A character string for the prefix of the resulting new
 #'  variables. See notes below.
 #' @param keep_original_cols A logical to keep the original variables in the
@@ -33,6 +33,7 @@
 #' @family multivariate transformation steps
 #' @export
 #' @details
+#'
 #' Principal component analysis (PCA) is a transformation of a
 #'  group of variables that produces a new set of artificial
 #'  features or components. These components are designed to capture
@@ -47,14 +48,11 @@
 #'  `options` argument or by using [step_center()]
 #'  and [step_scale()].
 #'
-#' The argument `num_comp` controls the number of components that
-#'  will be retained (the original variables that are used to derive
-#'  the components are removed from the data). The new components
-#'  will have names that begin with `prefix` and a sequence of
-#'  numbers. The variable names are padded with zeros. For example,
-#'  if `num_comp < 10`, their names will be `PC1` - `PC9`.
-#'  If `num_comp = 101`, the names would be `PC001` -
-#'  `PC101`.
+#' ```{r, echo = FALSE, results="asis"}
+#' prefix <- "PC"
+#' result <- knitr::knit_child("man/rmd/num_comp.Rmd")
+#' cat(result)
+#' ```
 #'
 #' Alternatively, `threshold` can be used to determine the
 #'  number of components that are required to capture a specified
@@ -62,9 +60,32 @@
 #'
 #' # Tidying
 #'
-#' When you [`tidy()`][tidy.recipe()] this step, use either `type = "coef"`
-#' for the variable loadings per component or `type = "variance"` for how
-#' much variance each component accounts for.
+#' When you [`tidy()`][tidy.recipe()] this step two things can happen depending
+#' the `type` argument. If `type = "coef"` a tibble returned with 4 columns
+#' `terms`, `value`, `component` , and `id`:
+#'
+#' \describe{
+#'   \item{terms}{character, the selectors or variables selected}
+#'   \item{value}{numeric, variable loading}
+#'   \item{component}{character, principle component}
+#'   \item{id}{character, id of this step}
+#' }
+#'
+#' If `type = "variance"` a tibble returned with 4 columns `terms`, `value`,
+#' `component` , and `id`:
+#'
+#' \describe{
+#'   \item{terms}{character, type of variance}
+#'   \item{value}{numeric, value of the variance}
+#'   \item{component}{integer, principle component}
+#'   \item{id}{character, id of this step}
+#' }
+#'
+#' ```{r, echo = FALSE, results="asis"}
+#' step <- "step_pca"
+#' result <- knitr::knit_child("man/rmd/tunable-args.Rmd")
+#' cat(result)
+#' ```
 #'
 #' @template case-weights-unsupervised
 #'
@@ -92,6 +113,7 @@
 #'
 #' tidy(pca_trans, number = 2)
 #' tidy(pca_estimates, number = 2)
+#' tidy(pca_estimates, number = 2, type = "variance")
 step_pca <- function(recipe,
                      ...,
                      role = "predictor",
@@ -105,10 +127,8 @@ step_pca <- function(recipe,
                      keep_original_cols = FALSE,
                      skip = FALSE,
                      id = rand_id("pca")) {
-  if (!is_tune(threshold) & !is_varying(threshold)) {
-    if (!is.na(threshold) && (threshold > 1 | threshold <= 0)) {
-      rlang::abort("`threshold` should be on (0, 1].")
-    }
+  if (!is_tune(threshold)) {
+    check_number_decimal(threshold, min = 0, max = 1, allow_na = TRUE)
   }
 
   add_step(
@@ -218,25 +238,22 @@ prep.step_pca <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_pca <- function(object, new_data, ...) {
+  check_new_data(object$columns, object, new_data)
+
   if (is.null(object$columns)) {
     object$columns <- stats::setNames(nm = rownames(object$res$rotation))
   }
 
-  if (length(object$columns) > 0 && !all(is.na(object$res$rotation))) {
-    check_new_data(object$columns, object, new_data)
-
-    pca_vars <- rownames(object$res$rotation)
-    comps <- scale(new_data[, pca_vars], object$res$center, object$res$scale) %*%
-      object$res$rotation
-    comps <- comps[, 1:object$num_comp, drop = FALSE]
-    comps <- check_name(comps, new_data, object)
-    new_data <- bind_cols(new_data, as_tibble(comps))
-    keep_original_cols <- get_keep_original_cols(object)
-
-    if (!keep_original_cols) {
-      new_data <- new_data[, !(colnames(new_data) %in% pca_vars), drop = FALSE]
-    }
+  if (length(object$columns) == 0 || all(is.na(object$res$rotation))) {
+    return(new_data)
   }
+
+  pca_vars <- rownames(object$res$rotation)
+  comps <- scale(new_data[, pca_vars], object$res$center, object$res$scale) %*%
+    object$res$rotation[, seq_len(object$num_comp), drop = FALSE]
+  comps <- check_name(comps, new_data, object)
+  new_data <- vec_cbind(new_data, as_tibble(comps))
+  new_data <- remove_original_cols(new_data, object, pca_vars)
   new_data
 }
 
@@ -251,7 +268,7 @@ print.step_pca <-
         title <- "No PCA components were extracted from "
         columns <- names(x$columns)
       } else {
-        title <- glue::glue("PCA extraction with ")
+        title <- glue("PCA extraction with ")
         columns <- rownames(x$res$rotation)
       }
     } else {
@@ -305,7 +322,7 @@ pca_variances <- function(x) {
     res <- tibble::tibble(
       terms = x,
       value = y,
-      component = rep(1:p, 4)
+      component = rep(seq_len(p), 4)
     )
   } else {
     res <- tibble::tibble(
