@@ -1,4 +1,4 @@
-#' Check Range Consistency
+#' Check range consistency
 #'
 #' `check_range` creates a *specification* of a recipe
 #'  check that will check if the range of a numeric
@@ -101,6 +101,7 @@ check_range_new <-
     )
   }
 
+#' @export
 prep.check_range <- function(x,
                              training,
                              info = NULL,
@@ -134,13 +135,22 @@ range_check_func <- function(x,
                              slack_prop = 0.05,
                              warn = FALSE,
                              colname = "x") {
-  stopifnot(
-    is.numeric(slack_prop),
-    is.numeric(x)
-  )
+  if (!is.numeric(x)) {
+    cli::cli_abort(
+      "{.arg x} must be a numeric vector, not {.obj_type_friendly {x}}."
+    )
+  }
+
+  if (!is.numeric(slack_prop)) {
+    cli::cli_abort(
+      "{.arg slack_prop} must be a numeric vector, \\
+      not {.obj_type_friendly {slack_prop}}."
+    )
+  }
+
   min_x <- min(x)
   max_x <- max(x)
-  msg <- NULL
+
   if (length(slack_prop) == 1) {
     lower_allowed <- lower - ((upper - lower) * slack_prop)
     upper_allowed <- upper + ((upper - lower) * slack_prop)
@@ -148,46 +158,49 @@ range_check_func <- function(x,
     lower_allowed <- lower - ((upper - lower) * slack_prop[1])
     upper_allowed <- upper + ((upper - lower) * slack_prop[2])
   } else {
-    rlang::abort("slack_prop should be of length 1 or of length 2")
+    cli::cli_abort(
+      "{.arg slack_prop} should be of length 1 or 2, not {length(slack_prop)}."
+    )
   }
 
-  if (min_x < lower_allowed & max_x > upper_allowed) {
-    msg <- paste0(
-      "min ", colname, " is ", min_x, ", lower bound is ",
-      lower_allowed, ", max x is ", max_x, ", upper bound is ",
-      upper_allowed
-    )
-  } else if (min_x < lower_allowed) {
-    msg <- paste0(
-      "min ", colname, " is ", min_x, ", lower bound is ",
-      lower_allowed
-    )
-  } else if (max_x > upper_allowed) {
-    msg <- paste0(
-      "max ", colname, " is ", max_x, ", upper bound is ",
-      upper_allowed
+  msg <- NULL
+  if (min_x < lower_allowed) {
+    msg <- c(
+      msg,
+      i = "Smallest value of {.var {colname}} is {min_x}, \\
+      crossing the lower bound {lower_allowed}."
     )
   }
-  if (warn & !is.null(msg)) {
-    rlang::warn(msg)
-  } else if (!is.null(msg)) {
-    rlang::abort(msg)
+  if (max_x > upper_allowed) {
+    msg <- c(
+      msg,
+      i = "Largest value of {.var {colname}} is {max_x}, \\
+      crossing the upper bound {upper_allowed}."
+    )
+  }
+
+  if (!is.null(msg)) {
+    if (warn) {
+      cli::cli_warn(msg)
+    } else {
+      cli::cli_abort(msg)
+    }
   }
 }
 
-bake.check_range <- function(object,
-                             new_data,
-                             ...) {
+#' @export
+bake.check_range <- function(object, new_data, ...) {
   col_names <- names(object$lower)
-  for (i in seq_along(col_names)) {
-    colname <- col_names[i]
+  check_new_data(col_names, object, new_data)
+
+  for (col_name in col_names) {
     range_check_func(
-      new_data[[colname]],
-      object$lower[colname],
-      object$upper[colname],
+      new_data[[col_name]],
+      object$lower[col_name],
+      object$upper[col_name],
       object$slack_prop,
       object$warn,
-      colname
+      col_name
     )
   }
   new_data
