@@ -1,7 +1,7 @@
-#' Zero Variance Filter
+#' Zero variance filter
 #'
-#' `step_zv` creates a *specification* of a recipe step
-#'  that will remove variables that contain only a single value.
+#' `step_zv()` creates a *specification* of a recipe step that will remove
+#' variables that contain only a single value.
 #'
 #' @inheritParams step_center
 #' @param removals A character string that contains the names of
@@ -18,8 +18,13 @@
 #'
 #' # Tidying
 #'
-#' When you [`tidy()`][tidy.recipe()] this step, a tibble with column
-#' `terms` (the columns that will be removed) is returned.
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble is returned with
+#' columns `terms` and `id`:
+#'
+#' \describe{
+#'   \item{terms}{character, names of the columns that will be removed}
+#'   \item{id}{character, id of this step}
+#' }
 #'
 #' @template case-weights-not-supported
 #'
@@ -125,9 +130,7 @@ prep.step_zv <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_zv <- function(object, new_data, ...) {
-  if (length(object$removals) > 0) {
-    new_data <- new_data[, !(colnames(new_data) %in% object$removals)]
-  }
+  new_data <- recipes_remove_cols(new_data, object)
   new_data
 }
 
